#include <ansi_c.h>
#include "ltr212clb_coeffs.h"
#include "localdefs.h"

#define EEPROM_COEFFS_ADDRESS 100

typedef unsigned short WORD;
typedef signed short SHORT;

typedef union{
	int value;
	unsigned char ubytes[4];
} int3;


INT LTR212_WriteEEPROM_UTS(descr_ltr212 *hnd, INT n, INT Address, BYTE *image);  
INT LTR212_ReadEEPROM_UTS(descr_ltr212 *hnd, INT n, INT Address, BYTE *image); 
INT LTR212_WriteFlashCRC_UTS(descr_ltr212 *hnd, WORD Flash_CRC);  
INT LTR212_EvaluateFlashCRC_UTS(descr_ltr212 *hnd, WORD *Flash_CRC);   


/*  4-     3  */
void pack3byte(int value, char* bytes) {
	int3 u;
	u.value = value;

	bytes[0] = u.ubytes[0];
	bytes[1] = u.ubytes[1];
	bytes[2] = u.ubytes[2];
}

/*  3   4-    */
int unpack3byte(char* bytes) {
	return (unsigned int)(*(unsigned int*)bytes) & 0x00FFFFFF;
}

/**
 *      .
 * \return      NULL   .
 */
LTR212clb_coeffs** create_coeffs_table() {
	LTR212clb_coeffs** result;
	int band, chan;

	/*   . */
	result = (LTR212clb_coeffs**)malloc(MAX_BAND*sizeof(LTR212clb_coeffs*));
	if (result==NULL) return NULL;

	/*   . */
	for (band=0;band<MAX_BAND;++band) {
		result[band] = (LTR212clb_coeffs*)malloc(MAX_CHANNEL*sizeof(LTR212clb_coeffs));
		if (result[band]==NULL) {
			for (--band;band>=0;++band) {
				free(result[band]);
			}
			free(result);
			return NULL;
		}
		for (chan=0;chan<MAX_CHANNEL;++chan) {
			result[band][chan].offset = 0x800000;
			result[band][chan].scale = 0x400000;
		}
	}

	return result;
}

/**
 *      .
 *
 */
void delete_coeffs_table(LTR212clb_coeffs** coeffs) {
	int band;
	
	for (band=0;band<MAX_BAND;++band) {
		free(coeffs[band]);
	}
	free(coeffs);
}

int read_coeffs(descr_ltr212 *h, LTR212clb_coeffs **coeffs) {
	char buffer[MAX_BAND*MAX_CHANNEL*3*2+1], *ptr;
	int band, chan;
	int res;

	res = LTR212_ReadEEPROM_UTS(h, sizeof(buffer)-1, EEPROM_COEFFS_ADDRESS, buffer);
	if (res) return res;

	ptr = buffer;
	for (band=0;band<MAX_BAND;++band) {
		for (chan=0;chan<MAX_CHANNEL;++chan) {
			coeffs[band][chan].offset = unpack3byte(ptr);
			ptr+=3;
			coeffs[band][chan].scale = (double)unpack3byte(ptr);
			ptr+=3;
		}
	}
	
	return res;
}

int write_coeffs(descr_ltr212 *h, LTR212clb_coeffs **coeffs) {
	char buffer[MAX_BAND*MAX_CHANNEL*3*2];
	char *ptr = buffer;
	int band, chan;
	int err;
	WORD Flash_CRC;

	for (band=0;band<MAX_BAND;++band) {
		for (chan=0;chan<MAX_CHANNEL;++chan) {
			pack3byte(coeffs[band][chan].offset, ptr);
			ptr+=3;
			pack3byte(coeffs[band][chan].scale, ptr);
			ptr+=3;
		}
	}

	err=LTR212_WriteEEPROM_UTS(h, sizeof(buffer), EEPROM_COEFFS_ADDRESS, buffer);
	if(err)
		return(err);

	Flash_CRC=0;
 
	err=LTR212_EvaluateFlashCRC_UTS(h, &Flash_CRC);//      .
	if(err)
		return(err);
   
   
	err=LTR212_WriteFlashCRC_UTS(h, Flash_CRC); //    .    
	return(err);
}


/**
 *           .
 *
 * \param h   LTR-212
 * \param band  
 *
 * \retutn  : 0 -- 
 */
int copy_coeffs(descr_ltr212 *h, int band) {
	char buffer[MAX_CHANNEL*3*2], DAC_data=0x20;
	int chan, res;
	WORD Flash_CRC;

	res = LTR212_ReadEEPROM_UTS(h, sizeof(buffer), EEPROM_COEFFS_ADDRESS+MAX_CHANNEL*3*2*band, buffer);
	if (res) return res;

	for (chan=0;chan<MAX_CHANNEL;++chan) {
		res = LTR212_WriteEEPROM_UTS(h, 6, 7*chan, buffer+6*chan);
		if (res) return res;
		res = LTR212_WriteEEPROM_UTS(h, 1, 7*chan+6, &DAC_data);
		if (res) return res;
	}

	Flash_CRC=0;
 
	res=LTR212_EvaluateFlashCRC_UTS(h, &Flash_CRC);//      .
	if(res)
		return(res);
   
   
	res=LTR212_WriteFlashCRC_UTS(h, Flash_CRC); //    .    
	return(res);
}

double value2code(double value, int band) {
	/*int shift = 0x800000 * (band / 4);*/
	int scale = 10 * (1 << (band % 4)) / (1 << (band / 4));
	return (int)(0x800000*value/scale/*-shift*/);
}

double code2value(double code, int band) {
	/*int shift = 0x800000 * (band / 4);*/
	int scale = 10 * (1 << (band % 4)) / (1 << (band / 4));
	return (code/*+shift*/)*scale/0x800000;
}

//#define SCALE_0 0x400000
#define SCALE_0 0x593CEA
#define NUMBER_A SCALE_0
#define NUMBER_B 0x400000

/**
*         
*  LTR-212   .     
*   .
*
* \param coeffs    
* \param band  
* \param phys_channel   (  1)
* \param max_etalon         .
* \param min_etalon         .
* \param max_val    LTR-212     .
* \param min_val    LTR-212     .
* \param zero_etalon         .
* \param zero_val    LTR-212     .
*
*/
int calc_coeffs(LTR212clb_coeffs **coeffs, int band, int phys_channel,
								double max_etalon, double min_etalon, double max_val, double min_val,
								double zero_etalon, double zero_val) {
	double val_code, etalon_code, scale;

	val_code = zero_val/*value2code(zero_val, band)*/;
	etalon_code = value2code(zero_etalon, band);
	scale = (max_etalon - min_etalon)/(code2value(max_val, band) - code2value(min_val, band));
	coeffs[band][phys_channel-1].scale = (int)(scale*SCALE_0+0.5);
	/*printf("calc_coeffs: %f %f %f %d\n", val_code, etalon_code, val_code - etalon_code,
				 (int)(val_code - etalon_code + 0x800000 + 0.5));*/
	/* :
	coeffs[band][phys_channel-1].offset = (int)(val_code - etalon_code/scale + 0x800000 + 0.5);
	*/
	if (band<4)
		coeffs[band][phys_channel-1].offset = (int)((val_code - etalon_code/scale)*NUMBER_B/NUMBER_A + 2*NUMBER_B + 0.5);
	else
		coeffs[band][phys_channel-1].offset =
			(int) ((val_code*scale - etalon_code /*+ 2*NUMBER_B*(scale-1)*/)*NUMBER_B/NUMBER_A/2 + 2*NUMBER_B);

	return 0;
}

/**
*    .
*
* \param coeffs    
* \param band  
* \param phys_channel   (  1)
* \param max_val    LTR-212  .
*
* \return    .
*/
double correct_value(LTR212clb_coeffs **coeffs, int band, int phys_channel, double value) {
	return code2value(correct_code(coeffs, band, phys_channel, value2code(value, band)), band);
}

/**
*    .
*
* \param coeffs    
* \param band  
* \param phys_channel   (  1)
* \param max_val    LTR-212  .
*
* \return     .
*/
double correct_code(LTR212clb_coeffs **coeffs, int band, int phys_channel, double code) {
	double result, scale;
	
	scale = (double)(coeffs[band][phys_channel-1].scale)/SCALE_0;
	if (band<4)
		result = scale * (code - (coeffs[band][phys_channel-1].offset - 2*NUMBER_B)*(double)NUMBER_A/NUMBER_B);
	else
		result = code*scale /*+ 2*NUMBER_B*(scale-1)*/
			- (coeffs[band][phys_channel-1].offset - 2*NUMBER_B)*2*(double)NUMBER_A/NUMBER_B ;

	return result;
}

